/*
 *  This is a sample plugin module
 *
 *  It can be compiled by any of the supported compilers:
 *
 *      - Borland C++, CBuilder, free C++
 *      - Watcom C++ for DOS32
 *      - Watcom C++ for OS/2
 *      - Visual C++
 *
 */

#include <ida.hpp>
#include <idp.hpp>
#include <bytes.hpp>
#include <loader.hpp>
#include <kernwin.hpp>



//--------------------------------------------------------------------------
// This callback is called for UI notification events
static int sample_callback(void * /*user_data*/, int event_id, va_list /*va*/)
{
  if ( event_id != ui_msg )     // avoid recursion
    if ( event_id != ui_setstate
      && event_id != ui_showauto
      && event_id != ui_refreshmarked ) // ignore uninteresting events
                    msg("ui_callback %d\n", event_id);
  return 0;                     // 0 means "process the event"
                                // otherwise the event would be ignored
}


//--------------------------------------------------------------------------
//
//      Initialize.
//
//      IDA will call this function only once.
//      If this function returns PLGUIN_SKIP, IDA will never load it again.
//      If this function returns PLUGIN_OK, IDA will unload the plugin but
//      remember that the plugin agreed to work with the database.
//      The plugin will be loaded again if the user invokes it by
//      pressing the hotkey or selecting it from the menu.
//      After the second load the plugin will stay on memory.
//      If this function returns PLUGIN_KEEP, IDA will keep the plugin
//      in the memory. In this case the initialization function can hook
//      into the processor module and user interface notification points.
//      See the hook_to_notification_point() function.
//
//      In this example we check the input file format and make the decision.
//      You may or may not check any other conditions to decide what you do:
//      whether you agree to work with the database or not.
//
int init(void)
{
  if ( inf.filetype == f_ELF ) return PLUGIN_SKIP;

// Please uncomment the following line to see how the notification works
//  hook_to_notification_point(HT_UI, sample_callback, NULL);

// Please uncomment the following line to see how the user-defined prefix works
//  set_user_defined_prefix(prefix_width, get_user_defined_prefix);

  return PLUGIN_KEEP;
}

//--------------------------------------------------------------------------
//      Terminate.
//      Usually this callback is empty.
//      The plugin should unhook from the notification lists if
//      hook_to_notification_point() was used.
//
//      IDA will call this function when the user asks to exit.
//      This function won't be called in the case of emergency exits.

void term(void)
{
  unhook_from_notification_point(HT_UI, sample_callback);
  set_user_defined_prefix(0, NULL);
}

//--------------------------------------------------------------------------
//
//      The plugin method
//
//      This is the main function of plugin.
//
//      It will be called when the user selects the plugin.
//
//              arg - the input argument, it can be specified in
//                    plugins.cfg file. The default is zero.
//
//

const short RADIO1_0000 = 0x0000;        // Group 1 First Radio Button
const short RADIO1_0001 = 0x0001;        // Group 1 Second Radio Button
const short RADIO1_0002 = 0x0002;        // Group 1 Third Radio Button
const short RADIO1_0003 = 0x0003;        // Group 1 Third Radio Button

const short RADIO2_0000 = 0x0000;        // Group 2 First Radio Button
const short RADIO2_0001 = 0x0001;        // Group 2 Second Radio Button
const short RADIO2_0002 = 0x0002;        // Group 2 Third Radio Button
const short RADIO2_0003 = 0x0003;        // Group 2 Third Radio Button

const short CHKBX1_0001 = 0x0001;        // Group 1 First Check Box
const short CHKBX1_0002 = 0x0002;        // Group 1 Second Check Box
const short CHKBX1_0004 = 0x0004;        // Group 1 Third Check Box
const short CHKBX1_0008 = 0x0008;        // Group 1 Fourth Check Box

const short CHKBX2_0001 = 0x0001;        // Group 2 First Check Box
const short CHKBX2_0002 = 0x0002;        // Group 2 Second Check Box
const short CHKBX2_0004 = 0x0004;        // Group 2 Third Check Box
const short CHKBX2_0008 = 0x0008;        // Group 2 Fourth Check Box




void run(int arg)
{

    short group1_radio = 0;
    short group1_chkbx = 0;

    short group2_radio = 0;
    short group2_chkbx = 0;


    // build the format string that creates the dialog.
    const char format[] =
    "STARTITEM 0\n"
    "HELP\n"                                                    // Help
    "The text seen in the 'help window' when the\n"             
    "user hits the 'Help' button.\n"
    "ENDHELP\n"

    "The Dialog Title\n"                                        // Title
    "This is the main text in the dialog.\n"                    // Dialog Text
    "(use mouse hover to show hints)\n\n"                       // Dialog Text

//First Group
    " Group #1\n"                                               // Dialog Text
     //  Group #1 Radio Buttons NOTE: using capital "R"
    "<#The Mouse Hover Hint for Radio Button 0.#"               // hint radio0
    "Text of Radio Button 0:R>"                                 // text radio0

    "<#The Mouse Hover Hint for Radio Button 1.#"               // hint radio1
    "Text of Radio Button 1:R>"                                 // text radio1

    "<#The Mouse Hover Hint for Radio Button 2.#"               // hint radio2
    "Text of Radio Button 2:R>"                                 // text radio2

     //NOTE the additional ">" at the end
    "<#The Mouse Hover Hint for Radio Button 3.#"               // hint radio3
    "Text of Radio Button 3:R>>"                                // text radio3

    
    // Group #1 Check Boxes NOTE: using capital "C"
    "<# The Mouse Hover Hint for Checkbox 1. #"                 // hint chkbx1
    "Frist Checkbox Tesxt (Mask 0x0001):C>"                     // mask 0x0001
    
    "<# The Mouse Hover Hint for Checkbox 2. #"                 // hint chkbx2
    "Second CheckBox Text (Mask 0x0002):C>"                     // mask 0x0002
    
    "<# The Mouse Hover Hint for Checkbox 3. #"                 // hint chkbx3
    "Third CheckBox Text (Mask 0x0004):C>"                      // mask 0x0004

    // NOTE the additional ">"
    "<# The Mouse Hover Hint for Checkbox 4. #"                 // hint chkbx4
    "Fourth CheckBox Text (Mask 0x0008):C>>\n\n\n\n\n"          // mask 0x0008

// Second Group
    " Group #2\n"                                               // Dialog Text
     //  Group #2 Radio Buttons NOTE: using lower case "r"
    "<#The Mouse Hover Hint for Radio Button 0.#"               // hint radio0
    "Text of Radio Button 0:r>"                                 // text radio0

    "<#The Mouse Hover Hint for Radio Button 1.#"               // hint radio1
    "Text of Radio Button 1:r>"                                 // text radio1

    "<#The Mouse Hover Hint for Radio Button 2.#"               // hint radio2
    "Text of Radio Button 2:r>"                                 // text radio2

     //NOTE the additional ">" at the end
    "<#The Mouse Hover Hint for Radio Button 3.#"               // hint radio3
    "Text of Radio Button 3:r>>"                                // text radio3

    
    // Group #2 Check Boxes NOTE: using lower case "c"
    "<# The Mouse Hover Hint for Checkbox 1. #"                 // hint chkbx1
    "Frist Checkbox Tesxt (Mask 0x0001):c>"                     // mask 0x0001
    
    "<# The Mouse Hover Hint for Checkbox 2. #"                 // hint chkbx2
    "Second CheckBox Text (Mask 0x0002):c>"                     // mask 0x0002
    
    "<# The Mouse Hover Hint for Checkbox 3. #"                 // hint chkbx3
    "Third CheckBox Text (Mask 0x0004):c>"                      // mask 0x0004

    // NOTE the additional ">"
    "<# The Mouse Hover Hint for Checkbox 4. #"                 // hint chkbx4
    "Fourth CheckBox Text (Mask 0x0008):c>>\n\n\n\n\n"          // mask 0x0008

    ; // End Dialog Format String

    
    // Set the group #1 default to have the second radio button selected
    group1_radio = RADIO1_0001;
    
    // Set the group #1 default to have the first and third check boxes checked
    group1_chkbx = CHKBX1_0001 + CHKBX1_0004;
    

    // Set the group #2 default to have the third radio button selected
    group2_radio = RADIO2_0002;
    
    // Set the group #2 default to have the second and fourth check boxes checked
    group2_chkbx = CHKBX2_0002 + CHKBX2_0008;

    // do the magic
    int ok = AskUsingForm_c( format, &group1_radio, &group1_chkbx, 
            &group2_radio, &group2_chkbx);

    // check return value
    if (!ok) 
        return;

    // and do what ever else you want e.g.:
    if(group1_chkbx & CHKBX1_0004)
        msg("The Third Check Box In The First Group Was Checked");


}

//--------------------------------------------------------------------------
char comment[] = "This is a sample plugin. It does nothing useful";

char help[] =
        "A sample plugin module\n"
        "\n"
        "This module shows you how to create plugin modules.\n"
        "\n"
        "It does nothing useful - just prints a message that is was called\n"
        "and shows the current address.\n";


//--------------------------------------------------------------------------
// This is the preferred name of the plugin module in the menu system
// The preferred name may be overriden in plugins.cfg file

char wanted_name[] = "UI Example2";


// This is the preferred hotkey for the plugin module
// The preferred hotkey may be overriden in plugins.cfg file
// Note: IDA won't tell you if the hotkey is not correct
//       It will just disable the hotkey.

char wanted_hotkey[] = "";


//--------------------------------------------------------------------------
//
//      PLUGIN DESCRIPTION BLOCK
//
//--------------------------------------------------------------------------

extern "C" plugin_t PLUGIN = {
  IDP_INTERFACE_VERSION,
  0,                    // plugin flags
  init,                 // initialize

  term,                 // terminate. this pointer may be NULL.

  run,                  // invoke plugin

  comment,              // long comment about the plugin
                        // it could appear in the status line
                        // or as a hint

  help,                 // multiline help about the plugin

  wanted_name,          // the preferred short name of the plugin
  wanted_hotkey         // the preferred hotkey to run the plugin
};
